@@ -1,8 +1,5 @@
 import os
 
-DEFAULT_REQUEST_TIMEOUT = 60 * 5  # 5 minutes
-DEFAULT_RESPONSE_TIMEOUT = 60 * 60  # 1 hour
-
 TEST_DATA_FILE = "test.yml"
 TRAIN_DATA_FILE = "train.yml"
 NLG_DATA_FILE = "responses.yml"
@@ -29,6 +26,9 @@ GLOBAL_USER_CONFIG_PATH = os.path.expanduser("~/.config/rasa/global.yml")
 DEFAULT_LOG_LEVEL_RASA_X = "WARNING"
 DEFAULT_LOG_LEVEL_LIBRARIES = "ERROR"
 ENV_LOG_LEVEL_LIBRARIES = "LOG_LEVEL_LIBRARIES"
+ENV_LOG_LEVEL_MATPLOTLIB = "LOG_LEVEL_MATPLOTLIB"
+ENV_LOG_LEVEL_RABBITMQ = "LOG_LEVEL_RABBITMQ"
+ENV_LOG_LEVEL_KAFKA = "LOG_LEVEL_KAFKA"
 
 DEFAULT_SANIC_WORKERS = 1
 ENV_SANIC_WORKERS = "SANIC_WORKERS"
