@@ -1,14 +1,27 @@
 import datetime
 import json
 import logging
-import requests
-from sanic import Blueprint, response
-from sanic.request import Request
+import re
+from http import HTTPStatus
 from typing import Text, Dict, Any, List, Iterable, Callable, Awaitable, Optional
 
-from rasa.core.channels.channel import UserMessage, OutputChannel, InputChannel
+import jwt
+import requests
+from jwt import InvalidKeyError, PyJWTError
+from jwt.algorithms import RSAAlgorithm
+from requests import HTTPError
+from sanic import Blueprint, response
+from sanic.request import Request
 from sanic.response import HTTPResponse
 
+from rasa.core.channels.channel import UserMessage, OutputChannel, InputChannel
+
+MICROSOFT_OPEN_ID_URI = (
+    "https://login.botframework.com/v1/.well-known/openidconfiguration"
+)
+
+BEARER_REGEX = re.compile(r"Bearer\s+(.*)")
+
 logger = logging.getLogger(__name__)
 
 MICROSOFT_OAUTH2_URL = "https://login.microsoftonline.com"
@@ -188,25 +201,93 @@ class BotFrameworkInput(InputChannel):
         self.app_id = app_id
         self.app_password = app_password
 
+        self.jwt_keys: Dict[Text, Any] = {}
+        self.jwt_update_time = datetime.datetime.fromtimestamp(0)
+
+        self._update_cached_jwk_keys()
+
+    def _update_cached_jwk_keys(self) -> None:
+        logger.debug("Updating JWT keys for the Botframework.")
+        response = requests.get(MICROSOFT_OPEN_ID_URI)
+        response.raise_for_status()
+        conf = response.json()
+
+        jwks_uri = conf["jwks_uri"]
+
+        keys_request = requests.get(jwks_uri)
+        keys_request.raise_for_status()
+        keys_list = keys_request.json()
+        self.jwt_keys = {key["kid"]: key for key in keys_list["keys"]}
+        self.jwt_update_time = datetime.datetime.now()
+
+    def _validate_jwt_token(self, jwt_token: Text) -> None:
+        jwt_header = jwt.get_unverified_header(jwt_token)  # type: ignore
+        key_id = jwt_header["kid"]
+        if key_id not in self.jwt_keys:
+            raise InvalidKeyError(f"JWT Key with ID {key_id} not found.")
+
+        key_json = self.jwt_keys[key_id]
+        public_key = RSAAlgorithm.from_jwk(key_json)  # type: ignore
+        jwt.decode(
+            jwt_token,
+            key=public_key,
+            audience=self.app_id,
+            algorithms=jwt_header["alg"],
+        )
+
+    def _validate_auth(self, auth_header: Optional[Text]) -> Optional[HTTPResponse]:
+        if not auth_header:
+            return response.text(
+                "No authorization header provided.", status=HTTPStatus.UNAUTHORIZED
+            )
+
+        # Update the JWT keys daily
+        if datetime.datetime.now() - self.jwt_update_time > datetime.timedelta(days=1):
+            try:
+                self._update_cached_jwk_keys()
+            except HTTPError as error:
+                logger.warning(
+                    f"Could not update JWT keys from {MICROSOFT_OPEN_ID_URI}."
+                )
+                logger.exception(error, exc_info=True)
+
+        auth_match = BEARER_REGEX.match(auth_header)
+        if not auth_match:
+            return response.text(
+                "No Bearer token provided in Authorization header.",
+                status=HTTPStatus.UNAUTHORIZED,
+            )
+
+        (jwt_token,) = auth_match.groups()
+
+        try:
+            self._validate_jwt_token(jwt_token)
+        except PyJWTError as error:
+            logger.error("Bot framework JWT token could not be verified.")
+            logger.exception(error, exc_info=True)
+            return response.text(
+                "Could not validate JWT token.", status=HTTPStatus.UNAUTHORIZED
+            )
+
+        return None
+
     @staticmethod
     def add_attachments_to_metadata(
         postdata: Dict[Text, Any], metadata: Optional[Dict[Text, Any]]
     ) -> Optional[Dict[Text, Any]]:
         """Merge the values of `postdata['attachments']` with `metadata`."""
-
         if postdata.get("attachments"):
             attachments = {"attachments": postdata["attachments"]}
             if metadata:
                 metadata.update(attachments)
             else:
                 metadata = attachments
-
         return metadata
 
     def blueprint(
         self, on_new_message: Callable[[UserMessage], Awaitable[Any]]
     ) -> Blueprint:
-
+        """Defines the Sanic blueprint for the bot framework integration."""
         botframework_webhook = Blueprint("botframework_webhook", __name__)
 
         @botframework_webhook.route("/", methods=["GET"])
@@ -215,6 +296,12 @@ class BotFrameworkInput(InputChannel):
 
         @botframework_webhook.route("/webhook", methods=["POST"])
         async def webhook(request: Request) -> HTTPResponse:
+            validation_response = self._validate_auth(
+                request.headers.get("Authorization")
+            )
+            if validation_response:
+                return validation_response
+
             postdata = request.json
             metadata = self.get_metadata(request)
 
