@@ -270,9 +270,7 @@ class SpacyNLP(GraphComponent):
         self, training_data: TrainingData, model: SpacyModel
     ) -> TrainingData:
         """Adds SpaCy tokens and features to training data messages."""
-        model = model.model
-
-        attribute_docs = self._docs_for_training_data(model, training_data)
+        attribute_docs = self._docs_for_training_data(model.model, training_data)
 
         for attribute in DENSE_FEATURIZABLE_ATTRIBUTES:
 
@@ -288,13 +286,12 @@ class SpacyNLP(GraphComponent):
 
     def process(self, messages: List[Message], model: SpacyModel) -> List[Message]:
         """Adds SpaCy tokens and features to messages."""
-        model = model.model
         for message in messages:
             for attribute in DENSE_FEATURIZABLE_ATTRIBUTES:
                 if message.get(attribute):
                     message.set(
                         SPACY_DOCS[attribute],
-                        self._doc_for_text(model, message.get(attribute)),
+                        self._doc_for_text(model.model, message.get(attribute)),
                     )
 
         return messages
