@@ -32,6 +32,7 @@ class Features:
         self.type = feature_type
         self.origin = origin
         self.attribute = attribute
+        self._cached_fingerprint: Optional[Text] = None
         if not self.is_dense() and not self.is_sparse():
             raise ValueError(
                 "Features must either be a numpy array for dense "
@@ -102,6 +103,7 @@ class Features:
         self.features = np.concatenate(
             (self.features, additional_features.features), axis=-1
         )
+        self._cached_fingerprint = None
 
     def _combine_sparse_features(self, additional_features: Features) -> None:
         from scipy.sparse import hstack
@@ -114,6 +116,7 @@ class Features:
             )
 
         self.features = hstack([self.features, additional_features.features])
+        self._cached_fingerprint = None
 
     def __key__(
         self,
@@ -148,16 +151,17 @@ class Features:
 
     def fingerprint(self) -> Text:
         """Calculate a stable string fingerprint for the features."""
-        if self.is_dense():
-            f_as_text = self.features.tostring()
-        else:
-            f_as_text = rasa.shared.nlu.training_data.util.sparse_matrix_to_string(
-                self.features
+        if self._cached_fingerprint is None:
+            if self.is_dense():
+                f_as_text = self.features.tobytes()
+            else:
+                f_as_text = rasa.shared.nlu.training_data.util.sparse_matrix_to_string(
+                    self.features
+                )
+            self._cached_fingerprint = rasa.shared.utils.io.deep_container_fingerprint(
+                [self.type, self.origin, self.attribute, f_as_text]
             )
-
-        return rasa.shared.utils.io.deep_container_fingerprint(
-            [self.type, self.origin, self.attribute, f_as_text]
-        )
+        return self._cached_fingerprint
 
     @staticmethod
     def filter(
